# -*- coding: UTF-8 -*-

"""
Communication (sending messages)
"""

# System imports
import django.template              as dtempl

# Project imports
import symonitoring_alert.constants            as const
from . import UserCommunicationBase


class UserCommunicationAlert(UserCommunicationBase):
    """Methods to communicate (alert) with a user"""

    __slots__ = (
        '_timeframe',
    )

    @property
    def timeframe(self):
        """:rtype: symonitoring_model.models.Timeframe"""
        return self._timeframe

    def __init__(
            self,
            logdata,
            user,
            timeframe,
            template=None,
            type_=const.TemplateType.ALERT
    ):
        # pylint: disable=too-many-arguments
        super(UserCommunicationAlert, self).__init__(
            logdata,
            user,
            template,
            type_
        )
        self._timeframe = timeframe

    def render_template(self, part=None, template=None, is_action=None):
        """Render template"""
        if not template:
            template = self.template
        try:
            tmpl = dtempl.Template(template[0])
            c    = dtempl.Context({
                'part'      : part,
                'user'      : self.user,
                'timeframe' : self.timeframe,
                'logdata'   : self.logdata,
                'alert'     : self.logdata.alert.model,
                'type'      : const.template_type_dict[self.type_].lower(),
                'is_action' : is_action,
            })
            res = tmpl.render(c).strip()
        except dtempl.TemplateSyntaxError as e:
            tmpl = dtempl.Template(template[1])
            c    = dtempl.Context({
                'part'      : part,
                'user'      : self.user,
                'timeframe' : self.timeframe,
                'logdata'   : self.logdata,
                'alert'     : self.logdata.alert.model,
                'type'      : const.template_type_dict[self.type_].lower(),
                'is_action' : is_action,
                'syntax'    : str(e),
            })
            res = tmpl.render(c).strip()
        return res
