# -*- coding: UTF-8 -*-

"""
Communication (sending messages)
"""

# System imports
import django.template              as dtempl

# Project imports
import symonitoring_alert.constants            as const
from . import UserCommunicationBase


class UserCommunicationInform(UserCommunicationBase):
    """Methods to communicate (alert) with a user"""

    def __init__(
            self,
            logdata,
            user,
            template=None,
            type_=const.TemplateType.INFOFEED,
    ):
        # pylint: disable=too-many-arguments
        super(UserCommunicationInform, self).__init__(
            logdata,
            user,
            template,
            type_
        )

    def render_template(self, part=None, template=None, is_action=None):
        """Render template"""
        if not template:
            template = self.template
        try:
            tmpl = dtempl.Template(template[0])
            c    = dtempl.Context({
                'part'      : part,
                'user'      : self.user,
                'logdata'   : self.logdata,
                'alert'     : self.logdata.alert.model,
                'type'      : const.template_type_dict[self.type_].lower(),
                'is_action' : is_action,
            })
            res = tmpl.render(c).strip()
        except dtempl.TemplateSyntaxError as e:
            tmpl = dtempl.Template(template[1])
            c    = dtempl.Context({
                'part'      : part,
                'user'      : self.user,
                'logdata'   : self.logdata,
                'alert'     : self.logdata.alert.model,
                'type'      : const.template_type_dict[self.type_].lower(),
                'is_action' : is_action,
                'syntax'    : str(e),
            })
            res = tmpl.render(c).strip()
        return res
