# -*- coding: UTF-8 -*-

"""
Dummy mode helper
"""

# System imports

# Project imports
import symonitoring_alert.common as cmn
import symonitoring_model.models as mod

s = cmn.get_settings()


class Dummy(cmn.ExecutorMixin):
    """Helper class for dummy methods."""

    def force_action(self, value):
        """Force the actiontype"""
        if not s.DUMMY_MODE:
            return value
        elif not s.DUMMY_FORCE:  # pragma: no cover
            return value
        else:
            return s.DUMMY_FORCE

    def dummy_team_users(self):
        """Gets dummy team users on executor."""
        dummy_team_users = mod.User.objects.filter(
            name__in=s.DUMMY_TEAM
        )
        if not dummy_team_users:  # pragma: no cover
            raise Exception(
                "Dummy team from settings not found in DB: %s" % str(
                    s.DUMMY_TEAM
                )
            )
        return dummy_team_users

    def force_team(self, value):
        """Forces the usasge of an emergency team."""
        if not s.DUMMY_MODE or not s.DUMMY_TEAM:
            return value
        else:
            users = self.dummy_team_users()
            return users

    @cmn.run_on_executor
    def dummy_inform_users(self):
        """Gets dummy inform users on executor."""
        dummy_inform_users = mod.User.objects.filter(
            name__in=s.DUMMY_INFORM
        )
        if not dummy_inform_users:  # pragma: no cover
            raise Exception(
                "Dummy inform users from settings not found in DB: %s" % str(
                    s.DUMMY_INFORM
                )
            )
        return dummy_inform_users

    @cmn.coroutine
    def force_inform_users(self, value):
        """Forces the usasge of infofeed recipients."""
        if not s.DUMMY_MODE or not s.DUMMY_INFORM:
            raise cmn.Return(value)
        else:
            users = yield self.dummy_inform_users()
            raise cmn.Return(users)
