# -*- coding: UTF-8 -*-
# pylint: disable=protected-access,missing-docstring

"""Test the dummy function"""

# System imports

# Project imports
import symonitoring_alert.common    as cmn
import symonitoring_alert.dummy     as dummy
import symonitoring_alert.constants as const
import symonitoring_model.models    as mod


class DummyTest(cmn.TestCase):
    def test_force_action_with_override(self):
        d = dummy.Dummy()
        self.assertEqual(
            d.force_action(const.ActionType.PHONE),
            const.ActionType.XMPP,
        )

    def test_force_action_no_override(self):
        d = dummy.Dummy()
        with cmn.disable_dummy_mode():
            self.assertEqual(
                d.force_action(const.ActionType.PHONE),
                const.ActionType.PHONE,
            )

    def test_force_team_no_dummy(self):
        with cmn._allow_block_for_test_only():
            d = dummy.Dummy()
            users = [mod.User.objects.all()[0]]
            self.assertAlmostEqual(
                d.force_team(users)[0].id,
                users[0].id,
            )

    def test_force_team_with_dummy(self):
        with cmn._allow_block_for_test_only():
            users = [mod.User.objects.all()[0]]
            users_other = [mod.User.objects.all()[1]]
            s = cmn.get_settings()
            old_team = s.DUMMY_TEAM
            try:
                d = dummy.Dummy()
                s.DUMMY_TEAM = (users_other[0].name, )
                self.assertEqual(
                    d.force_team(users)[0].id,
                    users_other[0].id,
                )
            finally:
                s.DUMMY_TEAM = old_team

    def test_inform_users_no_dummy_users(self):
        with cmn._allow_block_for_test_only():
            d = dummy.Dummy()
            user = [mod.User.objects.first()]

            @cmn.coroutine
            def coro():
                users = yield d.force_inform_users(user)
                raise cmn.Return(users)

            self.assertAlmostEqual(
                user,
                cmn.run_sync(coro)
            )

    def test_inform_users_with_dummy_users(self):
        with cmn._allow_block_for_test_only():
            d = dummy.Dummy()
            s = cmn.get_settings()
            old_users = s.DUMMY_TEAM
            try:
                user_real = [mod.User.objects.all()[0]]
                user_forced = [mod.User.objects.all()[1]]
                s.DUMMY_INFORM = (user_forced[0].name,)
                d.force_inform_users(user_real)

                @cmn.coroutine
                def coro_real():
                    users = yield d.force_inform_users(user_real)
                    raise cmn.Return(users)

                self.assertAlmostEqual(
                    user_forced,
                    list(cmn.run_sync(coro_real)),
                )

                @cmn.coroutine
                def coro_forced():
                    users = yield d.force_inform_users(user_forced)
                    raise cmn.Return(users)

                with cmn.disable_dummy_mode():
                    self.assertAlmostEqual(
                        user_forced,
                        list(cmn.run_sync(coro_forced)),
                    )
            finally:
                s.DUMMY_TEAM = old_users
