# -*- coding: UTF-8 -*-
# pylint: disable=missing-docstring

"""
Testing of handling alert holds
"""

# System imports

# Project imports
import symonitoring_alert.handle.hold_base_test as base
import symonitoring_alert.common                as cmn
import symonitoring_alert.constants             as const
import symonitoring_alert.handle.hold           as hhld
import symonitoring_alert.logdata               as ld


class HoldExecTest(base.HoldTestBase):
    """ Tests the execution of hold events (execution=accounting).
    """
    def test_handle_sync(self):
        logdata = self.run_init(
            notification=base.NOTIFICATION_HOST_DOWN,
            hostname="testhost25",
            now=base.NOW_MOCKED_OUT_OF_SLA,
        )
        self.assertIs(logdata.__class__, ld.LogData)
        self.assertEqual(
            logdata.log.processed,
            const.NagiosLog.STATE_NO_SLA
        )
        self.assertTrue(logdata.alert.model)
        self.assertEqual(
            logdata.alert.model.state,
            const.AlertState.STATE_DOWN
        )

        @cmn.coroutine
        def handle_hold_exec():
            handler = hhld.HoldExec()
            handler.logdata = logdata
            yield handler.handle()

        cmn.run_sync(handle_hold_exec)
        alert_holds = cmn.run_sync(self.coro_get_alert_holds)
        self.assertTrue(
            any(h.nagioslogid_id == logdata.log.id for h in alert_holds)
        )

    def test_handle_delete(self):
        logdata = self.run_init(
            notification=base.NOTIFICATION_HOST_DOWN,
            hostname="testhost25",
            now=base.NOW_MOCKED_IN_SLA,
        )
        self.assertIs(logdata.__class__, ld.LogData)
        self.assertEqual(
            logdata.log.processed,
            const.NagiosLog.STATE_ALERTED
        )
        self.assertTrue(logdata.alert.model)
        self.assertEqual(
            logdata.alert.model.state,
            const.AlertState.STATE_DOWN
        )

        @cmn.coroutine
        def handle_hold_exec():
            handler = hhld.HoldExec()
            logdata.log.processed = const.NagiosLog.STATE_ALERT_CLOSED
            handler.logdata = logdata
            yield handler.handle()

        cmn.run_sync(handle_hold_exec)
        alert_holds = cmn.run_sync(self.coro_get_alert_holds)
        self.assertFalse(
            any(h.nagioslogid_id == logdata.log.id for h in alert_holds)
        )
