# -*- coding: UTF-8 -*-

"""
Testing handling
"""

# pylint: disable=missing-docstring,too-many-lines,no-self-use
# pylint: disable=protected-access

# System imports
import datetime
import tornado.gen                             as gen
import mock
import django.db.utils                         as dbutils

# Project imports
import symonitoring_alert.common               as cmn
import symonitoring_alert.communication.alert  as sca
import symonitoring_alert.communication.inform as sci
import symonitoring_alert.communication        as scb
import symonitoring_alert.handle.nagioslog     as nl
import symonitoring_alert.handle.alert         as al
import symonitoring_alert.handle.hold          as hld
import symonitoring_alert.logdata              as ld
import symonitoring_model.models               as mod
import symonitoring_alert.constants            as const

notification_none_ok = {
    'address': u'127.0.0.1',
    'attempt': 1,
    'checkcommand': u'check-host-alive',
    'datetime': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    )),
    'duration': u'0d 0h 0m 0s',
    'hostname': u'localhost',
    'lastproblemid': 47,
    'name': u'localhost',
    'objecttype': u'NONE',
    'output': u'PING OK - Packet loss = 0%, RTA = 140.21 ms',
    'perfdata': u'rta=140.210007ms;3000.000000;5000.000000;0.000000 pl=0%;80;'
                '100;0',
    'problemid': 0,
    'processed': u'0',
    'state': u'UP',
    'statetype': u'HARD',
    'timestamp': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    ))
}

notification_host_ok = {
    'address': u'127.0.0.1',
    'attempt': 1,
    'checkcommand': u'check-host-alive',
    'datetime': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    )),
    'duration': u'0d 0h 0m 0s',
    'hostname': u'localhost',
    'lastproblemid': 0,
    'name': u'localhost',
    'objecttype': u'HOST',
    'output': u'PING OK - Packet loss = 0%, RTA = 140.21 ms',
    'perfdata': u'rta=140.210007ms;3000.000000;5000.000000;0.000000 pl=0%;80;'
                '100;0',
    'problemid': 1,
    'processed': u'0',
    'state': u'UP',
    'statetype': u'HARD',
    'timestamp': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    ))
}

notification_host_critical = {
    'address': u'192.24.12.3',
    'attempt': 1,
    'checkcommand': u'check-host-alive',
    'datetime': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    )),
    'duration': u'0d 0h 0m 0s',
    'hostname': u'localhost',
    'lastproblemid': 47,
    'name': u'localhost',
    'objecttype': u'HOST',
    'output': u'PING CRITICAL - Packet loss = 100%',
    'perfdata': u'rta=5000.000000ms;3000.000000;5000.000000;0.000000 pl=100%;'
                '80;100;0',
    'problemid': 48,
    'processed': u'0',
    'state': u'DOWN',
    'statetype': u'HARD',
    'timestamp': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    ))
}

notification_host_timefame_customer = {
    'address': u'192.24.12.3',
    'attempt': 1,
    'checkcommand': u'check-host-alive',
    'datetime': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    )),
    'duration': u'0d 0h 0m 0s',
    'hostname': u'testhost23',
    'lastproblemid': 47,
    'name': u'localhost',
    'objecttype': u'HOST',
    'output': u'PING CRITICAL - Packet loss = 100%',
    'perfdata': u'rta=5000.000000ms;3000.000000;5000.000000;0.000000 pl=100%;'
                '80;100;0',
    'problemid': 48,
    'processed': u'0',
    'state': u'DOWN',
    'statetype': u'HARD',
    'timestamp': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    ))
}

notification_host_wrong = {
    'address': u'192.24.12.3',
    'attempt': 1,
    'checkcommand': u'check-host-alive',
    'datetime': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    )),
    'duration': u'0d 0h 0m 0s',
    'hostname': u'asdfkajsdlf',
    'lastproblemid': 47,
    'name': u'localhost',
    'objecttype': u'HOST',
    'output': u'PING CRITICAL - Packet loss = 100%',
    'perfdata': u'rta=5000.000000ms;3000.000000;5000.000000;0.000000 pl=100%;'
                '80;100;0',
    'problemid': 48,
    'processed': u'0',
    'state': u'DOWN',
    'statetype': u'HARD',
    'timestamp': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    ))
}

notification_ok = {
    'address': '127.0.0.1',
    'attempt': 5,
    'checkcommand': 'tcp!22',
    'datetime': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    )),
    'duration': '0d 0h 0m 20',
    'hostname': 'testhost13',
    'lastproblemid': 18,
    'name': 'ssh__default',
    'objecttype': 'SERVICE',
    'output': 'TCP OK - 0.000 second response time on port 22',
    'perfdata': 'time=0.000154s;;;0.000000;10.000000',
    'problemid': 0,
    'processed': '0',
    'state': 'OK',
    'statetype': 'HARD',
    'timestamp': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    ))
}

notification_critical = {
    'address': '127.0.0.1',
    'attempt': 5,
    'checkcommand': 'tcp!221',
    'datetime': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    )),
    'duration': '0d 0h 4m 2s',
    'hostname': 'testhost7',
    'lastproblemid': 12,
    'name': 'ssh__default',
    'objecttype': 'SERVICE',
    'output': 'Connection refused',
    'perfdata': '',
    'problemid': 46,
    'processed': '0',
    'state': 'CRITICAL',
    'statetype': 'HARD',
    'timestamp': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    ))
}

notification_no_instance_in_string_critical = {
    'address': '127.0.0.1',
    'attempt': 5,
    'checkcommand': 'tcp!221',
    'datetime': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    )),
    'duration': '0d 0h 4m 2s',
    'hostname': 'testhost7',
    'lastproblemid': 12,
    'name': 'ssh',
    'objecttype': 'SERVICE',
    'output': 'Connection refused',
    'perfdata': '',
    'problemid': 46,
    'processed': '0',
    'state': 'CRITICAL',
    'statetype': 'HARD',
    'timestamp': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    ))
}

notification_soft = {
    'address': '127.0.0.1',
    'attempt': 4,
    'checkcommand': 'tcp!221',
    'datetime': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    )),
    'duration': '0d 0h 3m 4s',
    'hostname': 'testhost3',
    'lastproblemid': 23,
    'name': 'ssh__default',
    'objecttype': 'SERVICE',
    'output': 'Connection refused',
    'perfdata': '',
    'problemid': 42,
    'processed': '0',
    'state': 'CRITICAL',
    'statetype': 'SOFT',
    'timestamp': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    ))
}

notification_sla_critical = {
    'address': '127.0.0.1',
    'attempt': 5,
    'checkcommand': 'tcp!221',
    'datetime': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    )),
    'duration': '0d 0h 4m 2s',
    'hostname': 'testhost21',
    'lastproblemid': 12,
    'name': 'sshsla__default',
    'objecttype': 'SERVICE',
    'output': 'Connection refused',
    'perfdata': '',
    'problemid': 46,
    'processed': '0',
    'state': 'CRITICAL',
    'statetype': 'HARD',
    'timestamp': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    ))
}

notification_team_critical = {
    'address': '127.0.0.1',
    'attempt': 5,
    'checkcommand': 'tcp!221',
    'datetime': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    )),
    'duration': '0d 0h 4m 2s',
    'hostname': 'testhost22',
    'lastproblemid': 12,
    'name': 'ssh__default',
    'objecttype': 'SERVICE',
    'output': 'Connection refused',
    'perfdata': '',
    'problemid': 46,
    'processed': '0',
    'state': 'CRITICAL',
    'statetype': 'HARD',
    'timestamp': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    ))
}

notification_host_team_critical = {
    'address': u'192.24.12.3',
    'attempt': 1,
    'checkcommand': u'check-host-alive',
    'datetime': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    )),
    'duration': u'0d 0h 0m 0s',
    'hostname': u'testhost22',
    'lastproblemid': 47,
    'name': u'localhost',
    'objecttype': u'HOST',
    'output': u'PING CRITICAL - Packet loss = 100%',
    'perfdata': u'rta=5000.000000ms;3000.000000;5000.000000;0.000000 pl=100%;'
                '80;100;0',
    'problemid': 93,
    'processed': u'0',
    'state': u'DOWN',
    'statetype': u'HARD',
    'timestamp': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    ))
}

notification_host_team_ok = {
    'address': u'127.0.0.1',
    'attempt': 1,
    'checkcommand': u'check-host-alive',
    'datetime': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    )),
    'duration': u'0d 0h 0m 0s',
    'hostname': u'testhost22',
    'lastproblemid': 93,
    'name': u'localhost',
    'objecttype': u'HOST',
    'output': u'PING OK - Packet loss = 0%, RTA = 140.21 ms',
    'perfdata': u'rta=140.210007ms;3000.000000;5000.000000;0.000000 pl=0%;80;'
                '100;0',
    'problemid': 1,
    'processed': u'0',
    'state': u'UP',
    'statetype': u'HARD',
    'timestamp': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    ))
}

notification_soft = {
    'address': '127.0.0.1',
    'attempt': 5,
    'checkcommand': 'tcp!22',
    'datetime': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    )),
    'duration': '0d 0h 0m 20',
    'hostname': 'testhost13',
    'lastproblemid': 18,
    'name': 'ssh__default',
    'objecttype': 'SERVICE',
    'output': 'TCP OK - 0.000 second response time on port 22',
    'perfdata': 'time=0.000154s;;;0.000000;10.000000',
    'problemid': 0,
    'processed': '0',
    'state': 'OK',
    'statetype': 'SOFT',
    'timestamp': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    ))
}

notification_team_srv_ok = {
    'address': '127.0.0.1',
    'attempt': 5,
    'checkcommand': 'tcp!22',
    'datetime': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    )),
    'duration': '0d 0h 0m 20',
    'hostname': 'testhost22',
    'lastproblemid': 81,
    'name': 'ssh__default',
    'objecttype': 'SERVICE',
    'output': 'TCP OK - 0.000 second response time on port 22',
    'perfdata': 'time=0.000154s;;;0.000000;10.000000',
    'problemid': 0,
    'processed': '0',
    'state': 'OK',
    'statetype': 'HARD',
    'timestamp': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    ))
}

notification_team_srv_critical = {
    'address': '127.0.0.1',
    'attempt': 5,
    'checkcommand': 'tcp!221',
    'datetime': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    )),
    'duration': '0d 0h 4m 2s',
    'hostname': 'testhost22',
    'lastproblemid': 12,
    'name': 'ssh__default',
    'objecttype': 'SERVICE',
    'output': 'Connection refused',
    'perfdata': '',
    'problemid': 81,
    'processed': '0',
    'state': 'CRITICAL',
    'statetype': 'HARD',
    'timestamp': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    ))
}

notification_team_srv_warning = {
    'address': '127.0.0.1',
    'attempt': 5,
    'checkcommand': 'tcp!221',
    'datetime': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    )),
    'duration': '0d 0h 4m 2s',
    'hostname': 'testhost22',
    'lastproblemid': 12,
    'name': 'ssh__default',
    'objecttype': 'SERVICE',
    'output': 'Connection refused',
    'perfdata': '',
    'problemid': 81,
    'processed': '0',
    'state': 'WARNING',
    'statetype': 'HARD',
    'timestamp': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    ))
}


class HandleTest(cmn.TestCase):
    # pylint: disable=too-many-public-methods

    def handle_log(self, object_, no_exec=True):
        """Handle a single mod.NagiosLog object

        :rtype: nl.NagiosLog an list of Alerts
        :returns: The created nagios log object and a list of generated Alerts
        """
        handlers, alerts = self.handle_log_list([object_], no_exec)
        return handlers[0], alerts

    def handle_log_list(self, object_list, no_exec=True):
        """Handle a list of mod.NagiosLog objects

        :rtype: list of nl.NagiosLog and list of created Alerts
        :rval: The created nagios log objects and list of generated Alerts
        """
        handlers = []
        for object_ in object_list:
            handler = nl.NagiosLog()
            handlers.append(handler)
            self.save_log(object_)
            self._handle_log_call(object_, handler, no_exec)
        alerts = self.get_alerts()
        return handlers, alerts

    def _handle_log_call(self, model, handler, no_exec):
        """Use tornado to execute the handler on a NagiosLog object"""
        @gen.coroutine
        def coro_handle():
            yield handler.handle(model, no_exec=no_exec)
        cmn.run_sync(coro_handle)

    def handle_alert(self, alert, logdata):

        @gen.coroutine
        def coro():
            alert.logdata = logdata
            yield alert.handle()

        cmn.run_sync(coro)

    @cmn.run_on_executor
    def get_alerts_ex(self):
        """Return all available alerts, run the query on the executor"""
        return list(mod.Alert.objects.select_related().order_by('-id'))

    def get_alerts(self):
        """Return all available alerts"""
        return cmn.run_sync(self.get_alerts_ex)

    def test_host(self):
        log = mod.Nagioslog(
            **notification_host_critical
        )
        self.save_log(log)
        with mock.patch.object(log, "save") as msave:
            handler, _ = self.handle_log(log)
            self.assertEqual(
                log.processed,
                const.NagiosLog.STATE_NO_SLA
            )
            self.assertTrue(msave.called)
            self.assertIsNone(handler.logdata.service)
            self.assertTrue(handler.logdata.is_host)
            self.assertEqual(
                handler.logdata.host.customerid.name,
                "Testkunde",
            )
            self.assertEqual(
                handler.logdata.sla.name,
                "No Alarm",
            )

    def test_service(self):
        log = mod.Nagioslog(
            **notification_critical
        )
        self.save_log(log)
        with mock.patch.object(log, "save") as msave:
            handler, _ = self.handle_log(log)
            self.assertEqual(log.processed, const.NagiosLog.STATE_NO_SLA)
            self.assertTrue(msave.called)
            self.assertIsNotNone(handler.logdata.service)
            self.assertTrue(handler.logdata.is_service)
            self.assertEqual(
                handler.logdata.host.customerid.name,
                "Testkunde",
            )
            self.assertEqual(
                handler.logdata.sla.name,
                "No Alarm",
            )
            self.assertEqual(
                handler.logdata.service.name,
                "ssh",
            )

    def test_none(self):
        log = mod.Nagioslog(
            **notification_none_ok
        )
        self.save_log(log)
        with mock.patch.object(log, "save") as msave:
            self.handle_log(log)
            self.assertEqual(
                log.processed,
                const.NagiosLog.STATE_NO_PREVIOUS_DOWN
            )
            self.assertTrue(msave.called)

    def test_soft(self):
        log = mod.Nagioslog(
            **notification_soft
        )
        self.save_log(log)
        with mock.patch.object(log, "save") as msave:
            handler, _ = self.handle_log(log)
            self.assertTrue(msave.called)
            self.assertTrue(handler.logdata.is_soft)

    def test_timeframe_customer(self):
        log = mod.Nagioslog(
            **notification_team_critical
        )
        self.save_log(log)
        with mock.patch.object(log, "save") as msave:
            handler, _ = self.handle_log(log)
            self.assertTrue(msave.called)
            self.assertTrue(msave.called)
            self.assertGreater(len(handler.logdata.users), 0)

    def test_wrong_host(self):
        log = mod.Nagioslog(
            **notification_host_wrong
        )
        self.save_log(log)
        with mock.patch.object(log, "save") as msave:
            self.handle_log(log)
            self.assertTrue(msave.called)
            self.assertEqual(
                log.processed,
                const.NagiosLog.STATE_MALFORMED_ROW
            )

    def test_no_instance_in_string(self):
        log = mod.Nagioslog(
            **notification_no_instance_in_string_critical
        )
        self.save_log(log)
        with mock.patch.object(log, "save") as msave:
            self.handle_log(log)
            self.assertTrue(msave.called)
            self.assertEqual(
                log.processed,
                const.NagiosLog.STATE_MALFORMED_ROW
            )

    def test_sla(self):
        log = mod.Nagioslog(
            **notification_sla_critical
        )
        self.save_log(log)
        with mock.patch.object(log, "save") as msave:
            handler, _ = self.handle_log(log)
            self.assertEqual(log.processed, const.NagiosLog.STATE_NO_TEAM)
            self.assertTrue(msave.called)
            self.assertIsNotNone(handler.logdata.host.slaid_id)
            self.assertIsNotNone(handler.logdata.service.slaid_id)

    def test_timeframe(self):
        log = mod.Nagioslog(
            **notification_sla_critical
        )
        self.save_log(log)
        with mock.patch.object(log, "save") as msave:
            handler, _ = self.handle_log(log)
            self.assertEqual(log.processed, const.NagiosLog.STATE_NO_TEAM)
            self.assertTrue(msave.called)
            self.assertIsNotNone(handler.logdata.host.slaid_id)
            self.assertIsNotNone(handler.logdata.service.slaid_id)
        self.assertTrue(handler.logdata.check_timeframe())

    def test_hostdown(self):
        log = mod.Nagioslog(
            **notification_sla_critical
        )
        self.save_log(log)
        with mock.patch.object(log, "save") as msave:
            handler, _ = self.handle_log(log)
            self.assertEqual(log.processed, const.NagiosLog.STATE_NO_TEAM)
            self.assertTrue(msave.called)
            self.assertIsNotNone(handler.logdata.host.slaid_id)
            self.assertIsNotNone(handler.logdata.service.slaid_id)
            self.assertFalse(handler.logdata.check_hostdown())

    def test_sync_throws_exception(self):
        exceptions = [
            mod.Alert.DoesNotExist,
            dbutils.IntegrityError,
        ]
        for exception in exceptions:
            log = mod.Nagioslog(
                **notification_team_critical
            )
            self.save_log(log)
            patch_sync_ex = mock.patch(
                "symonitoring_alert.logdata.LogData.sync_ex",
                side_effect=exception
            )
            patch_report_error = mock.patch(
                "symonitoring_alert.communication.UserCommunicationBase."
                "report_error"
            )
            patch_schedule_timeout = mock.patch(
                "symonitoring_alert.handle.alert.AlertExec."
                "schedule_next_timeout"
            )
            patch_dispatch = mock.patch(
                "symonitoring_alert.handle.alert.AlertExec."
                "dispatch_action"
            )
            patch_timeout = mock.patch(
                "symonitoring_alert.common."
                "add_timeout"
            )
            patch_callback = mock.patch(
                "symonitoring_alert.common."
                "add_callback"
            )
            with patch_sync_ex, \
                    patch_report_error, \
                    patch_schedule_timeout as malert_schedule, \
                    patch_dispatch as malert_dispatch, \
                    patch_timeout as add_timeout, \
                    patch_callback:
                self.handle_log(log, False)
                self.assertFalse(malert_schedule.called)
                self.assertFalse(malert_dispatch.called)
                self.assertTrue(add_timeout.called)
                cmn.run_sync(add_timeout.call_args[0][1])
            self.assertEqual(
                log.processed,
                const.NagiosLog.STATE_DOES_NOT_EXIST
            )

    def test_team_false(self):
        log = mod.Nagioslog(
            **notification_sla_critical
        )
        self.save_log(log)
        with mock.patch.object(
            log, "save"
        ) as msave, mock.patch(
            "symonitoring_alert.communication."
            "UserCommunicationBase.report_error"
        ):
            handler, _ = self.handle_log(log)
            self.assertEqual(log.processed, const.NagiosLog.STATE_NO_TEAM)
            self.assertTrue(msave.called)
            self.assertIsNotNone(handler.logdata.host.slaid_id)
            self.assertIsNotNone(handler.logdata.service.slaid_id)
        with cmn._allow_block_for_test_only():
            self.assertFalse(handler.logdata.check_team())

    def test_team_true(self):
        log = mod.Nagioslog(
            **notification_team_critical
        )
        self.save_log(log)
        with mock.patch.object(
            log, "save"
        ) as msave, mock.patch(
            "symonitoring_alert.communication."
            "UserCommunicationBase.report_error"
        ) as mreport_error:
            handler, alerts = self.handle_log(log)
            self.assertEqual(1, len(alerts))
            self.assertEqual(log.processed, const.NagiosLog.STATE_ALERTED)
            self.assertTrue(msave.called)
            self.assertFalse(mreport_error.called)
            self.assertIsNotNone(handler.logdata.host.slaid)
        with cmn._allow_block_for_test_only():
            self.assertTrue(handler.logdata.check_team())

    def test_team_hostdown(self):
        logs = [
            mod.Nagioslog(
                **notification_host_team_critical
            ),
            mod.Nagioslog(
                **notification_team_critical
            ),
        ]
        for log in logs:
            self.save_log(log)
        handlers, alerts = self.handle_log_list(logs)
        self.assertEqual(1, len(alerts))
        self.assertIsNotNone(handlers[-1].logdata.host.slaid_id)
        self.assertTrue(handlers[-1].logdata.check_hostdown())

    def test_team_hostdown_after(self):
        logs = [
            mod.Nagioslog(
                **notification_team_critical
            ),
            mod.Nagioslog(
                **notification_host_team_critical
            ),
        ]
        handlers = []
        for log in logs:
            self.save_log(log)

        @gen.coroutine
        def coro():
            for log in logs:
                handler = nl.NagiosLog()
                yield handler.handle(log, no_exec=True)
                handlers.append(handler)

                hold_handler = hld.HoldExec()
                logdata = ld.LogData(log)
                try:
                    yield logdata.load_ex()
                except ld.MalformedRow:
                    logdata = None
                if logdata:
                    hold_handler.logdata = logdata
                    yield hold_handler.handle()

        cmn.run_sync(coro)
        alerts = self.get_alerts()
        self.assertEqual(2, len(alerts))
        self.assertTrue(alerts[1].acknowledgedautomatically)
        self.assertEqual(
            alerts[1].comment,
            "Automatically closed because of new alert."
        )

    def test_recovery(self):
        logs = [
            mod.Nagioslog(
                **notification_host_team_critical
            ),
            mod.Nagioslog(
                **notification_host_team_ok
            ),
        ]
        for log in logs:
            self.save_log(log)
        handlers, alerts = self.handle_log_list(logs)
        self.assertEqual(1, len(alerts))
        alert = alerts[0]
        assert isinstance(alert, mod.Alert)
        self.assertTrue(alert.acknowledged)
        self.assertTrue(alert.acknowledgedautomatically)
        self.assertTrue(handlers[0].logdata.host.hastrouble)
        self.assertFalse(handlers[1].logdata.host.hastrouble)

    def test_recovery_service(self):
        logs = [
            mod.Nagioslog(
                **notification_team_srv_critical
            ),
            mod.Nagioslog(
                **notification_team_srv_ok
            ),
        ]
        for log in logs:
            self.save_log(log)
        handlers, alerts = self.handle_log_list(logs)
        self.assertEqual(1, len(alerts))
        alert = alerts[0]
        assert isinstance(alert, mod.Alert)
        self.assertTrue(alert.acknowledged)
        self.assertTrue(alert.acknowledgedautomatically)
        self.assertTrue(handlers[0].logdata.host.hastrouble)
        self.assertFalse(handlers[1].logdata.host.hastrouble)
        self.assertTrue(handlers[0].logdata.service_instance.hastrouble)
        self.assertFalse(handlers[1].logdata.service_instance.hastrouble)

    def test_downtime(self):
        log = mod.Nagioslog(
            **notification_team_critical
        )
        self.save_log(log)
        with mock.patch.object(log, "save") as msave:
            handler, alerts = self.handle_log(log)
            self.assertEqual(1, len(alerts))
            self.assertTrue(msave.called)
        handler.logdata._downtimes = [
            self.create_downtime(handler.logdata, cmn.now())
        ]
        self.assertTrue(handler.logdata.check_downtime())

    def test_downtime_later(self):
        log = mod.Nagioslog(
            **notification_team_critical
        )
        self.save_log(log)
        with mock.patch.object(log, "save") as msave:
            handler, alerts = self.handle_log(log)
            self.assertTrue(msave.called)
            self.assertEqual(1, len(alerts))
        handler.logdata._downtimes = [
            self.create_downtime(
                handler.logdata,
                cmn.now() + datetime.timedelta(days=2)
            )
        ]
        self.assertFalse(handler.logdata.check_downtime())

    def test_downtime_earlier(self):
        log = mod.Nagioslog(
            **notification_team_critical
        )
        self.save_log(log)
        with mock.patch.object(log, "save") as msave:
            handler, alerts = self.handle_log(log)
            self.assertTrue(msave.called)
            self.assertEqual(1, len(alerts))
        handler.logdata._downtimes = [
            self.create_downtime(
                handler.logdata, cmn.now() - datetime.timedelta(days=2)
            )
        ]
        self.assertTrue(handler.logdata.check_downtime())

    def test_downtime_earlier_count(self):
        log = mod.Nagioslog(
            **notification_team_critical
        )
        self.save_log(log)
        with mock.patch.object(log, "save") as msave:
            handler, alerts = self.handle_log(log)
            self.assertTrue(msave.called)
            self.assertEqual(1, len(alerts))
        handler.logdata._downtimes = [
            self.create_downtime_count(
                handler.logdata, cmn.now() - datetime.timedelta(days=4)
            )
        ]
        self.assertFalse(handler.logdata.check_downtime())

    def test_downtime_earlier_count_inc(self):
        log = mod.Nagioslog(
            **notification_team_critical
        )
        self.save_log(log)
        with mock.patch.object(log, "save") as msave:
            handler, alerts = self.handle_log(log)
            self.assertTrue(msave.called)
            self.assertEqual(1, len(alerts))
        handler.logdata._downtimes = [
            self.create_downtime_count(
                handler.logdata, cmn.now() - datetime.timedelta(days=1)
            )
        ]
        self.assertTrue(handler.logdata.check_downtime())

    def test_downtime_from_to(self):
        log = mod.Nagioslog(
            **notification_team_critical
        )
        self.save_log(log)
        with mock.patch.object(log, "save") as msave:
            handler, alerts = self.handle_log(log)
            self.assertTrue(msave.called)
            self.assertEqual(1, len(alerts))
        handler.logdata._downtimes = [
            self.create_downtime_from_to(handler.logdata, cmn.now())
        ]
        self.assertTrue(handler.logdata.check_downtime())

    def test_downtime_to_from(self):
        log = mod.Nagioslog(
            **notification_team_critical
        )
        self.save_log(log)
        with mock.patch.object(log, "save") as msave:
            handler, alerts = self.handle_log(log)
            self.assertTrue(msave.called)
            self.assertEqual(1, len(alerts))
        handler.logdata._downtimes = [
            self.create_downtime_to_from(handler.logdata, cmn.now())
        ]
        self.assertTrue(handler.logdata.check_downtime())

    def create_downtime(self, logdata, dt):
        """Create a downtime for a host"""
        assert isinstance(logdata, ld.LogData)
        datetimefrom = dt - datetime.timedelta(minutes=1)
        datetimeto   = None
        return mod.Downtime(
            hostid=logdata.host,
            datetimefrom=datetimefrom,
            datetimeto=datetimeto,
            durationhours=1.0,
            rrule="FREQ=DAILY;INTERVAL=2;",
        )

    def create_downtime_count(self, logdata, dt):
        """Create a downtime for a host"""
        assert isinstance(logdata, ld.LogData)
        datetimefrom = dt - datetime.timedelta(minutes=1)
        datetimeto   = None
        return mod.Downtime(
            hostid=logdata.host,
            datetimefrom=datetimefrom,
            datetimeto=datetimeto,
            durationhours=1.0,
            rrule="FREQ=DAILY;INTERVAL=1;COUNT=2;",
        )

    def create_downtime_from_to(self, logdata, dt):
        """Create a downtime for a host"""
        assert isinstance(logdata, ld.LogData)
        datetimefrom = dt - datetime.timedelta(minutes=1)
        datetimeto   = dt + datetime.timedelta(hours=1)
        return mod.Downtime(
            hostid=logdata.host,
            datetimefrom=datetimefrom,
            datetimeto=datetimeto
        )

    def create_downtime_to_from(self, logdata, dt):
        """Create a downtime for a host"""
        assert isinstance(logdata, ld.LogData)
        datetimefrom = dt + datetime.timedelta(hours=1)
        datetimeto   = dt - datetime.timedelta(minutes=1)
        return mod.Downtime(
            hostid=logdata.host,
            datetimefrom=datetimefrom,
            datetimeto=datetimeto
        )

    def test_team_exec(self):
        log = mod.Nagioslog(
            **notification_team_critical
        )
        self.save_log(log)
        with mock.patch.object(log, "save") as msave:
            with mock.patch(
                "symonitoring_alert.common.add_timeout"
            ) as cll, mock.patch(
                "symonitoring_alert.common.add_callback"
            ):
                handler, alerts = self.handle_log(log, False)
                self.assertEqual(1, len(alerts))
                self.assertEqual(log.processed, const.NagiosLog.STATE_ALERTED)
                self.assertTrue(msave.called)
                self.assertIsNotNone(handler.logdata.host.slaid)
                self.assertTrue(cll.called)

    def test_alert_exec(self):
        alert = al.AlertExec()
        log = mod.Nagioslog(
            **notification_team_critical
        )
        self.save_log(log)
        with mock.patch("symonitoring_alert.common.add_timeout") as tmout:
            with mock.patch.multiple(
                alert,
                mktimeout=lambda w, x, y, z: 1
            ):
                assert isinstance(tmout, mock.Mock)
                handler, _ = self.handle_log(log)
                self.handle_alert(alert, handler.logdata)
                self.assertTrue(tmout.called)
                tmout.assert_any_call(
                    datetime.timedelta(minutes=0),
                    1
                )

    def test_alert_warning(self):
        alert = al.AlertExec()
        log = mod.Nagioslog(
            **notification_team_srv_warning
        )
        self.save_log(log)
        patched_add_timeout   = mock.patch(
            "symonitoring_alert.common.add_timeout"
        )
        patched_display_alert = mock.patch(
            "symonitoring_alert.handle.alert.AlertInit.display_alert"
        )
        with patched_add_timeout as mtimeout, patched_display_alert as mdis:
            handler, _ = self.handle_log(log)
            self.handle_alert(alert, handler.logdata)
            self.assertFalse(mtimeout.called)
            self.assertTrue(mdis.called)

    def test_alert_display(self):
        alert = al.AlertExec()
        log = mod.Nagioslog(
            **notification_team_srv_warning
        )
        self.save_log(log)
        patched_add_timeout   = mock.patch(
            "symonitoring_alert.common.add_timeout"
        )
        patched_display_alert = mock.patch(
            "symonitoring_alert.handle.alert.AlertInit.display_alert"
        )
        with patched_add_timeout as mtimeout, patched_display_alert as mdis:
            handler, _ = self.handle_log(log)
            self.handle_alert(alert, handler.logdata)
            self.assertFalse(mtimeout.called)
            self.assertTrue(mdis.called)

    def test_alert_email(self):
        handler = nl.NagiosLog()
        alert   = al.AlertExec()
        log = mod.Nagioslog(
            **notification_team_critical
        )
        self.save_log(log)

        @gen.coroutine
        def coro():
            yield handler.handle(log, no_exec=True)

            alert.logdata = handler.logdata
            timeframe     = alert.logdata.timeframes[0]
            user          = handler.logdata.users[timeframe][0]
            yield alert.handle_EMAIL(timeframe, user)

        with mock.patch("smtplib.SMTP") as smtp:
            assert isinstance(smtp, mock.Mock)
            cmn.run_sync(coro)
            self.assertTrue(smtp.called)
            self.assertTrue(smtp.return_value.sendmail.called)
            self.assertTrue(smtp.return_value.quit.called)

    @cmn.coroutine
    def coro_false(self):
        raise cmn.Return(False)

    def test_alert_sms(self):
        handler = nl.NagiosLog()
        alert   = al.AlertExec()
        log = mod.Nagioslog(
            **notification_team_critical
        )
        self.save_log(log)

        @gen.coroutine
        def coro():
            yield handler.handle(log, no_exec=True)

            alert.logdata = handler.logdata
            timeframe     = alert.logdata.timeframes[0]
            user          = handler.logdata.users[timeframe][0]
            yield alert.handle_SMS(timeframe, user)

        with mock.patch(
            "symonitoring_alert.communication.UserCommunicationBase.send_sms"
        ) as comm:
            assert isinstance(comm, mock.Mock)
            comm.return_value = self.coro_false()
            cmn.run_sync(coro)
            self.assertTrue(comm.called)

    def test_alert_dispatch(self):
        handler = nl.NagiosLog()
        alert   = al.AlertExec()
        log = mod.Nagioslog(
            **notification_team_critical
        )
        self.save_log(log)

        @gen.coroutine
        def coro():
            yield handler.handle(log, no_exec=True)

            logdata       = handler.logdata
            alert.logdata = logdata
            timeframe     = logdata.timeframes[0]
            user          = logdata.users[timeframe][0]
            rule          = timeframe.ext_rules[0]
            yield alert.dispatch_action(
                timeframe,
                rule,
                logdata.users,
                user,
            )

        with mock.patch(
            "symonitoring_alert.communication.UserCommunicationBase.send_xmpp"
        ) as comm:
            assert isinstance(comm, mock.Mock)
            comm.return_value = self.coro_false()
            cmn.run_sync(coro)
            self.assertTrue(comm.called)

    def test_alert_dispatch_phone(self):
        handler = nl.NagiosLog()
        alert   = al.AlertExec()
        log = mod.Nagioslog(
            **notification_team_critical
        )
        self.save_log(log)

        @gen.coroutine
        def coro():
            yield handler.handle(log, no_exec=True)

            logdata       = handler.logdata
            alert.logdata = logdata
            timeframe     = logdata.timeframes[0]
            user          = logdata.users[timeframe][0]
            rule          = timeframe.ext_rules[2]
            yield alert.dispatch_action(
                timeframe,
                rule,
                logdata.users,
                user,
            )

        with mock.patch(
            "symonitoring_alert.communication.UserCommunicationBase.call_phone"
        ) as comm:
            assert isinstance(comm, mock.Mock)
            comm.return_value = self.coro_false()
            with cmn.disable_dummy_mode():
                cmn.run_sync(coro)
            self.assertTrue(comm.called)

    def test_alert_phone(self):
        handler = nl.NagiosLog()
        alert   = al.AlertExec()
        log = mod.Nagioslog(
            **notification_team_critical
        )
        self.save_log(log)

        @gen.coroutine
        def coro():
            yield handler.handle(log, no_exec=True)

            alert.logdata = handler.logdata
            timeframe     = alert.logdata.timeframes[0]
            user          = handler.logdata.users[timeframe][0]
            yield alert.handle_PHONE(timeframe, user)

        with mock.patch(
            "symonitoring_alert.communication.UserCommunicationBase.call_phone"
        ) as comm:
            assert isinstance(comm, mock.Mock)
            comm.return_value = self.coro_false()
            cmn.run_sync(coro)
            self.assertTrue(comm.called)

    @classmethod
    def get_handler_with_logdata(cls):
        handler = nl.NagiosLog()
        log = mod.Nagioslog(
            **notification_team_critical
        )
        cls.save_log(log)

        @gen.coroutine
        def coro():
            yield handler.handle(log, no_exec=True)

        cmn.run_sync(coro)
        return handler

    def test_render_template_alert(self):
        handler   = self.get_handler_with_logdata()
        logdata   = handler.logdata
        timeframe = handler.logdata.timeframes[0]
        user      = handler.logdata.users[timeframe][0]
        comm      = sca.UserCommunicationAlert(
            logdata=logdata,
            timeframe=timeframe,
            user=user,
        )
        tmp = comm.render_template(
            template=("{{ timeframe.id }}",),
        )
        self.assertEqual(tmp, "1")
        tmp = comm.render_template(
            template=("{{ logdata.host.hostname }}",),
        )
        self.assertEqual(tmp, "testhost22")
        tmp = comm.render_template(
            template=("{{ user.email }}",),
        )
        self.assertEqual(tmp, "test@localhost")
        tmp = comm.render_template(
            template=("""
{{ user.email }}
{% if syntax = "huhu" %
{{ syntax }}
{% endif %}""", """
{% if syntax %}{{ syntax }}{% endif %}"""),
        )
        self.assertEqual(tmp, 'Invalid block tag: &#39;endif&#39;')

    def test_render_template_inform(self):
        handler   = self.get_handler_with_logdata()
        logdata   = handler.logdata
        timeframe = handler.logdata.timeframes[0]
        user      = handler.logdata.users[timeframe][0]
        comm      = sci.UserCommunicationInform(
            logdata=logdata,
            user=user,
        )
        tmp = comm.render_template(
            template=("{{ logdata.host.hostname }}",),
        )
        self.assertEqual(tmp, "testhost22")
        tmp = comm.render_template(
            template=("{{ user.email }}",),
        )
        self.assertEqual(tmp, "test@localhost")
        tmp = comm.render_template(
            template=("""
{{ user.email }}
{% if syntax = "huhu" %
{{ syntax }}
{% endif %}""", """
{% if syntax %}{{ syntax }}{% endif %}"""),
        )
        self.assertEqual(tmp, 'Invalid block tag: &#39;endif&#39;')

    def test_mktimeout(self):
        handler = nl.NagiosLog()
        log = mod.Nagioslog(
            **notification_team_critical
        )
        self.save_log(log)
        # pylint: disable=attribute-defined-outside-init
        self.mkcalled = False

        @gen.coroutine
        def coro2():
            self.mkcalled = True

        @gen.coroutine
        def coro():
            yield handler.handle(log)
        with mock.patch(
            "symonitoring_alert.common.add_timeout"
        ) as cllbck, mock.patch(
            "symonitoring_alert.common.add_callback"
        ):
            cmn.run_sync(coro)
            self.assertTrue(cllbck.called)
            handler = cllbck.call_args[0][1]
        with mock.patch(
            "symonitoring_alert.alert.Alert"
            ".create_update_notification_datetime"
        ) as update:
            with mock.patch("symonitoring_alert.common.add_timeout") as cllbck:
                try:
                    # pylint: disable=assignment-from-no-return
                    update.return_value = coro2()
                    timeout_sec = scb.xmpp._timeout_seconds
                    scb.xmpp._timeout_seconds = 0.6
                    cmn.run_sync(handler)
                    self.assertTrue(update.called)
                    self.assertTrue(self.mkcalled)
                finally:
                    scb.xmpp._timeout_seconds = timeout_sec

    def test_acknowledged_duplicated_alert(self):
        handler = nl.NagiosLog()
        log1 = mod.Nagioslog(
            **notification_team_critical
        )
        self.save_log(log1)
        log2 = mod.Nagioslog(
            **notification_team_critical
        )
        self.save_log(log2)

        @gen.coroutine
        def coro_log1():
            yield handler.handle(log1)

        @gen.coroutine
        def coro_log2():
            yield handler.handle(log2)

        with mock.patch(
            "symonitoring_alert.common.add_timeout"
        ), mock.patch(
            "symonitoring_alert.common.add_callback"
        ):
            # New alert
            cmn.run_sync(coro_log1)
            alerts = self.get_alerts()
            self.assertEqual(1, len(alerts))
            alert = alerts[0]
            # Alert gets acknowledged
            with cmn._allow_block_for_test_only():
                alert.acknowledged = True
                alert.datetimeacknowledged = cmn.now()
                alert.save()
            # There is a new critical message for the same host/service,
            # assure old alert gets closed appropiately
            cmn.run_sync(coro_log2)
            alerts = self.get_alerts()
            self.assertEqual(2, len(alerts))
            # "Reload" alert from db as state has changed
            old_alert = alerts[1]
            new_alert = alerts[0]
            self.assertIsNotNone(old_alert.datetimerecovery)
            self.assertEqual(
                old_alert.comment,
                u'Automatically closed because of new alert.'
            )
            with cmn._allow_block_for_test_only():
                nagioslogs = old_alert.nagioslog_set.all()
                self.assertEqual(len(nagioslogs), 1)
            nagioslog = nagioslogs[0]
            self.assertEqual(
                int(nagioslog.processed),
                const.NagiosLog.STATE_ALERTED
            )
            self.assertIsNone(new_alert.datetimerecovery)
            self.assertIsNone(new_alert.comment)
            with cmn._allow_block_for_test_only():
                nagioslogs = new_alert.nagioslog_set.all()
                self.assertEqual(len(nagioslogs), 1)
            nagioslog = nagioslogs[0]
            self.assertEqual(
                int(nagioslog.processed),
                const.NagiosLog.STATE_ALERTED
            )
