# -*- coding: UTF-8 -*-

"""
Handle hold of informative events
"""

# System imports

# Project imports
import symonitoring_alert.common        as cmn
import symonitoring_alert.hold.base     as hold_base
import symonitoring_model.models        as mod


class HoldInform(hold_base.HoldBase):
    """Class for handling the hold of informative events."""

    def get_or_create(self):
        """Puts a NagiosLog entry on hold from being informed."""

        (self._hold, created) = mod.HoldInform.objects.get_or_create(
            nagioslogid_id=self.logdata.log.id
        )
        cmn.lg.debug(
            'Setting/Creating hold inform %d for logid %d (created: %s)',
            self._hold.id,
            self.logdata.log.id,
            created
        )

    def delete(self):
        """Removes all NagiosLog entries matching given entry from being on
        hold."""

        mod.HoldInform.objects.filter(
            nagioslogid_id=self.logdata.log.id
        ).delete()
        cmn.lg.debug(
            'Deleting hold inform for logid %d',
            self.logdata.log.id
        )

    def get(self):
        """Gets all entries being on hold matching given NagiosLog entry."""

        try:
            cmn.lg.debug(
                "Getting hold inform for logdata %d",
                self.logdata.log.id
            )
            return mod.HoldInform.objects.get(
                nagioslogid_id=self.logdata.log.id
            )
        except mod.HoldInform.DoesNotExist:
            return None

    @classmethod
    def set(cls, log_id):
        """Puts the given NagiosLog entry on hold from being processed."""

        (hold, created) = mod.HoldInform.objects.get_or_create(
            nagioslogid_id=log_id
        )
        cmn.lg.debug(
            'Saved hold %d for logid %d (created: %s)',
            hold.id,
            log_id,
            created
        )

    @cmn.run_on_executor
    def related_active_entries_ex(self):
        """ Run related_active_entries() on executor """
        return self.related_active_entries()

    def related_active_entries(self):
        """ Get all related nagios log entries """

        log = self.logdata.log
        return mod.HoldInform.objects.select_related().filter(
            nagioslogid__hostname=log.hostname,
            nagioslogid__name=log.name
        ).order_by('-nagioslogid_id')
