# -*- coding: UTF-8 -*-

"""
Outlaw blocking calls during testing
"""

# System imports
import threading

# Project imports

_main_thread = threading.current_thread()
disabled = False


def raise_django_blocking():
    """Patch blocking methods in django orm to raise if called
    in main thread"""
    import django.db.models.sql.compiler as compiler
    import inspect

    def raise_blocking(function):
        """Outlaw blocking calls in main-thread"""
        orig_func = function

        def raiser_func(*args, **kwargs):
            """Wrapper raises if we are in main thread"""
            if _main_thread == threading.current_thread():
                if not disabled:
                    raise IOError("django may not block in main thread.")
            return orig_func(*args, **kwargs)
        return raiser_func

    for _, obj in inspect.getmembers(compiler):
        if inspect.isclass(obj):
            if hasattr(obj, "execute_sql"):
                obj.execute_sql = raise_blocking(
                    obj.execute_sql
                )


def install():
    """Call all raise patch methods"""
    raise_django_blocking()
