# -*- coding: UTF-8 -*-

"""
Quering handling
"""

# pylint: disable=missing-docstring

# System imports
import datetime
import tornado.gen               as gen
import mock

# Project imports
import symonitoring_alert.common            as cmn
import symonitoring_model.models            as mod
import symonitoring_alert.handle.nagioslog  as nl
import symonitoring_alert.query             as query

notification_host_team_critical = {
    'address': u'192.24.12.3',
    'attempt': 1,
    'checkcommand': u'check-host-alive',
    'datetime': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    )),
    'duration': u'0d 0h 0m 0s',
    'hostname': u'testhost22',
    'lastproblemid': 47,
    'name': u'localhost',
    'objecttype': u'HOST',
    'output': u'PING CRITICAL - Packet loss = 100%',
    'perfdata': u'rta=5000.000000ms;3000.000000;5000.000000;0.000000 pl=100%;'
                '80;100;0',
    'problemid': 93,
    'processed': u'0',
    'state': u'DOWN',
    'statetype': u'HARD',
    'timestamp': cmn.local_datetime(datetime.datetime(
        2014, 5, 6, 15, 19, 7
    ))
}


class QueryTest(cmn.TestCase):

    def test_restart(self):
        handler = nl.NagiosLog()
        log = mod.Nagioslog(
            **notification_host_team_critical
        )
        self.save_log(log)

        @gen.coroutine
        def coro():
            yield handler.handle(log, no_exec=True)
        cmn.run_sync(coro)

        nagioslog = query.NagiosLog()

        @gen.coroutine
        def coro2():
            yield nagioslog.initialize_alerts()

        with mock.patch("symonitoring_alert.common.add_timeout") as cllbck:
            cmn.run_sync(coro2)
            self.assertTrue(cllbck.called)
