# -*- coding: UTF-8 -*-

"""
Django settings for syalert project.

For more information on this file, see
https://docs.djangoproject.com/en/1.6/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.6/ref/settings/
"""

# Use pymysql as mysqldb
# the maintainer of pymsql keeps 100% mysqldb compatibility
# and runs unittests against django
try:
    import pymysql
    pymysql.install_as_MySQLdb()
except ImportError:  # pragma: no cover
    pass

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os

BASE_DIR = os.path.dirname(os.path.dirname(__file__))

# System import
try:
    import configparser as cp
except ImportError:
    import ConfigParser as cp

# Project imports
import symonitoring_alert.logger    as sylg

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.6/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '6r&$s)&u0p*%f*wfki6gy-p(p1&8@$(54(#r(n-o&@zu8(7tiq'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = False

TEMPLATE_DEBUG = False

ALLOWED_HOSTS = []

# Database
# https://docs.djangoproject.com/en/1.6/ref/settings/#databases


INSTALLED_APPS = [
    'symonitoring_model',
    'symonitoring_alert',
]
MIDDLEWARE_CLASSES = []

# Internationalization
# https://docs.djangoproject.com/en/1.6/topics/i18n/
LANGUAGE_CODE = 'en-us'
USE_I18N = True
USE_L10N = True
DEFAULT_CHARSET = 'utf-8'

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.6/howto/static-files/
STATIC_URL = '/static/'

# SyAlert

SETTINGS_FILE     = '/var/lib/symonitoring/config/syalert.ini'
C                 = cp.ConfigParser()
C.read(SETTINGS_FILE)
TMP_DIR           = '/tmp/syalert'
LOCK_FILE         = '/var/lock/syalert'

DATABASES = {
    'default' : {
        'ENGINE'   : 'django.db.backends.mysql',
        'NAME'     : C.get('DATABASE', 'NAME'),
        'USER'     : C.get('DATABASE', 'USER'),
        'PASSWORD' : C.get('DATABASE', 'PASSWORD'),
        'HOST'     : C.get('DATABASE', 'HOST'),
    }
}
if 'MYSQL_PWD' in os.environ:
    DATABASES['default']['PASSWORD'] = os.environ['MYSQL_PWD']

# Testing
DUMMY_MODE         = False
DUMMY_FORCE        = 4
DUMMY_TEAM         = []
DUMMY_INFORM       = []

# Basic
SMTP_HOST          = C.get('SMTP', 'HOST')
SMTP_PORT          = C.getint('SMTP', 'PORT')
SENDER             = C.get('SMTP', 'SENDER')
WORKERS            = C.getint('GENERAL', 'WORKERS')
POLL_RATE          = C.getint('RATES', 'POLL') * 1000
HOLD_RATE          = C.getint('RATES', 'HOLD') * 1000
XMPP_CHECK_RATE    = C.getint('RATES', 'XMPP_CHECK') * 1000
COLLECT_RATE       = C.getint('RATES', 'COLLECT') * 1000
DEPENDENCY_TIMEOUT = C.getint('TIMEOUTS', 'DEPENDENCY') * 1000
TIMING_TEST_SCALE  = 2.0          # Used to scale timing tests

# Logging
LOGGING_LEVEL      = C.get('LOGGING', 'LEVEL')
if LOGGING_LEVEL == 'CRITICAL':
    LG             = sylg.setup_logger(sylg.ERROR, stderr=False)
else:
    LG             = sylg.setup_logger(sylg.DEBUG, stderr=False)
LOOP_LOG           = 0
LOG_SQL            = False

# SyAlert SMS
ASPSMS             = C.getboolean('ASPSMS', 'ENABLED')
ASPSMS_URL         = C.get('ASPSMS', 'URL')
ASPSMS_USERKEY     = C.get('ASPSMS', 'USERKEY')
ASPSMS_PASSWORD    = C.get('ASPSMS', 'PASSWORD')
ASPSMS_ORIGINATOR  = C.get('ASPSMS', 'ORIGINATOR')

GSM_SPOOL          = C.get('GSM', 'SPOOL')

# SyAlert Phone
PBX_BANNER         = C.get('PBX', 'BANNER')
PBX_IP             = C.get('PBX', 'IP')
PBX_PORT           = C.getint('PBX', 'PORT')
PBX_USER           = C.get('PBX', 'USER')
PBX_PW             = C.get('PBX', 'PASSWORD')
PBX_CONTEXT        = C.get('PBX', 'CONTEXT')
PBX_CHANNEL        = C.get('PBX', 'CHANNEL')
PBX_DEST           = C.get('PBX', 'DEST')
PBX_CALLERID       = C.get('PBX', 'CALLERID')

# SyAlert XMPP
XMPP_USER          = C.get('XMPP', 'USER')
XMPP_PASSWORD      = C.get('XMPP', 'PASSWORD')
XMPP_ADDRESS       = C.get('XMPP', 'ADDRESS')

# SyAlert MATTERMOST
WEBHOOK = {
    'url': C.get('MATTERMOST', 'HOOK'),
    'username': C.get('MATTERMOST', 'USER'),
}

# Timezone
USE_TZ             = False
TIME_ZONE          = 'Europe/Zurich'
